#!/usr/bin/env python

############################################################################
#
# MODULE:       r.surf.ba.py 0.1 (alpha)
# AUTHOR(S):    Ivan Marchesini <ivan.marchesini AT irpi.cnr.it>
# PURPOSE:      2D interpolation of geologic bedding attitudes
# COPYRIGHT:    (C) 2012 Ivan Marchesini & Michele Santangelo
#
#               This program is free software under the GNU General Public
#               License (>=v2). Read the file COPYING that comes with GRASS
#               for details.
#
#############################################################################

#%module
#% description: 2D interpolation of geologic bedding attitudes
#% keywords: bedding attitudes
#% keywords: geology, bedding attitude
#%end
#%option
#% key: ba
#% type: string
#% gisprompt: old,vector,vector
#% description: name of the bedding attitude vector layer
#% required: yes
#%end
#%option
#% key: inclination
#% type: string
#~ #% gisprompt: old,raster,dcell
#% description: name of the attribute field containing inclination values
#% required: yes
#% answer: slope50
#%end
#%option
#% key: dipdirection
#% type: string
#% description: name of the attribute field containing dipdirection values
#% required: yes
#% answer: azimuth
#%end
#%option
#% key: polarity
#% type: string
#% description: name of the attribute field containing 1 (upside strata) and 2 (upside-down strata)
#% required: yes
#% answer: polarity
#%end
#%option
#% key: dtm
#% type: string
#% gisprompt: old,raster,dcell
#% description: name of the dtm
#% required: yes
#%end
#%option
#% key: out_inc
#% type: string
#% gisprompt: new,dcell,raster
#% description: name of the inclination raster map
#% required: yes
#%end
#%option
#% key: out_imm
#% type: string
#% gisprompt: new,dcell,raster
#% description: name of the dipdirection raster map
#% required: yes
#%end
#%option
#% key: out_cosmap
#% type: string
#% gisprompt: new,dcell,raster
#% description: name of the directional coseno raster map
#% required: yes
#%end
#%option
#% key: out_morphostructural
#% type: string
#% gisprompt: new,dcell,raster
#% description: name of the directional coseno raster map
#% required: yes
#%end


import sys
import os
import atexit
import grass.script as grass
from grass.script import core as grasscore
import time
import datetime
from math import *


start=datetime.datetime.now()

def cleanup():
  pass

def main():
    
  ba = options['ba']
  incl = options['inclination']
  imm = options['dipdirection']
  pol = options['polarity']
  dtm = options['dtm']  
  inc_map = options['out_inc']
  imm_map = options['out_imm']
  cos_map = options['out_cosmap']
  morphostructural_map = options['out_morphostructural']
    
  grass.run_command("v.db.addcolumn", map=ba, layer=1, columns="x double precision, y double precision, z double precision")

  where=pol+"=1"
  value="sin(radians(%s))*sin(radians(%s))" % (incl,imm)
  grass.run_command("v.db.update", map=ba, column="x", value=value, where=where) 
  value="sin(radians(%s))*cos(radians(%s))" %(incl,imm)
  grass.run_command("v.db.update", map=ba, column="y", value=value, where=where)  
  value="cos(radians(%s))" % incl
  grass.run_command("v.db.update", map=ba, column="z", value=value, where=where)    

  where=pol+"=2"
  value="sin(radians(%s))*(-sin(radians(%s)))" % (incl,imm)
  grass.run_command("v.db.update", map=ba, column="x", value=value, where=where) 
  value="sin(radians(%s))*(-cos(radians(%s)))" %(incl,imm)
  grass.run_command("v.db.update", map=ba, column="y", value=value, where=where)  
  value="-cos(radians(%s))" % incl
  grass.run_command("v.db.update", map=ba, column="z", value=value, where=where)    

  #grass.run_command("g.region", vect=ba, rast=dtm)
  grass.run_command("g.region", rast=dtm)
  
  grass.mapcalc("${out} = ${A}/${A}",
  out = "MASK",
  A = dtm, overwrite=True) 
  
  grass.run_command("v.surf.rst", input=ba, elev="X_rst", zcolumn="x", overwrite=True)
  grass.run_command("v.surf.rst", input=ba, elev="Y_rst", zcolumn="y", overwrite=True)
  grass.run_command("v.surf.rst", input=ba, elev="Z_rst", zcolumn="z", overwrite=True)
 
 
  grass.mapcalc("${out} = sqrt((${A}^2)+(${B}^2)+(${C}^2))",
  out = "N_rst",
  A = "X_rst",
  B = "Y_rst",
  C = "Z_rst", overwrite=True)  
  
    
  grass.mapcalc("${out} = if(${A}>=0,acos(${A}/${B}),180-acos(-(${A})/${B}))",
  out = "incl_rst",
  A = "Z_rst",
  B = "N_rst", overwrite=True)
  
  print("ciao")
  
  grass.mapcalc("${out} = sqrt(${A}^2+${B}^2)",
  out = "M_rst",
  A = "X_rst",
  B = "Y_rst", overwrite=True)
                                         
    #~ """${out} = if(${C}>=0,  \                  
                               #~ (if(${A}>=0 && ${B}>=0,asin(${A}/M_rst), \# se siamo nel primo quadrante (NE)
                                   #~ if(${A}<0 && ${B}>=0,360-asin(-(${A}/M_rst)),  \# se siamo nel secondo quadrante (NO)
                                      #~ if(${A}<0 && ${B}<0,180+asin(-(${A}/M_rst)), \# se siamo nel terzo quadrante (SO)
                                         #~ if(${A}>=0 && ${B}<0,180-asin(${A}/M_rst)))))), \#se siamo nel quarto quadrante (SE) e chiudo il caso in cui ${A}>0
                                #~ #se Z_srt<0 (cioe se ho strati rovesci)
                               #~ (if(${A}>=0 && ${B}>=0,asin(${A}/M_rst)+180,  \# se siamo nel primo quadrante
                                  #~ if(${A}<0 && ${B}>=0,360-asin(-(${A}/M_rst))-180,  \# se siamo nel secondo quadrante
                                     #~ if(${A}<0 && ${B}<0,180+asin(-(${A}/M_rst))-180, \# se siamo nel terzo quadrante
                                         #~ if(${A}>=0 && ${B}<0,180-asin(${A}/M_rst)+180))))) \#se siamo nel quarto quadrante """ 
  

  grass.mapcalc("${out} = if(${C} >= 0,(if(${A} >= 0 && ${B} >= 0,asin(${A}/M_rst),if(${A} < 0 && ${B} >= 0,360-asin(-(${A}/M_rst)),if(${A} < 0 && ${B} < 0,180+asin(-(${A}/M_rst)),if(${A} >= 0 && ${B} < 0,180-asin(${A}/M_rst)))))),(if(${A} >= 0 && ${B} >= 0,asin(${A}/M_rst)+180,if(${A} <0 && ${B} >= 0,360-asin(-(${A}/M_rst))-180,if(${A} < 0 && ${B} < 0,180+asin(-(${A}/M_rst))-180,if(${A} >= 0 && ${B} < 0,180-asin(${A}/M_rst)+180))))))",
  out = "imm_rst",
  A = "X_rst",
  B = "Y_rst", 
  C = "Z_rst", overwrite=True)

  #CalCOLO TOBIA
  grass.run_command("r.slope.aspect", elevation=dtm, slope="tmp_slope", aspect="tmp_aspect", overwrite=True)
  
  grass.mapcalc("${out} = if(${A} > 0 && ${A} < 90,-(${A}-90),null())",
  out = "tmp_aspect1",
  A = "tmp_aspect", overwrite=True)
  
  grass.mapcalc("${out} = if(${A} > 0 && ${A} < 90,-(${A}-90),null())",
  out = "tmp_aspect1",
  A = "tmp_aspect", overwrite=True)  
  
  grass.mapcalc("${out} = if(${A} > 270 && ${A} < 360 ,(-(${A}-360)+90),null())",
  out = "tmp_aspect2",
  A = "tmp_aspect", overwrite=True)   
  
  grass.mapcalc("${out} = if(${A} > 180 && ${A} < 270 ,(-(${A}-360)+90),null())",
  out = "tmp_aspect3",
  A = "tmp_aspect", overwrite=True)    
  
  grass.mapcalc("${out} = if(${A} > 90 && ${A} < 180 ,(-(${A}-360)+90),null())",
  out = "tmp_aspect4",
  A = "tmp_aspect", overwrite=True)    


  grass.run_command("r.null", map="tmp_aspect1", null="0")
  grass.run_command("r.null", map="tmp_aspect2", null="0")
  grass.run_command("r.null", map="tmp_aspect3", null="0")
  grass.run_command("r.null", map="tmp_aspect4", null="0")
  

  
  grass.mapcalc("${out} = ${A}+${B}+${C}+${D}",
  out = "tmp_aspect_tot",
  A = "tmp_aspect1",
  B = "tmp_aspect2",
  C = "tmp_aspect3",
  D = "tmp_aspect4", overwrite=True)  
    
  grass.mapcalc("${out} = cos(${A}-${B})",
  out = "cosmap",
  A = "imm_rst",
  B = "tmp_aspect_tot", overwrite=True)  

  grass.mapcalc("${out} = cos(${A})*cos(${B})+sin(${A})*sin(${B})*cos(${C}-${D})",
  out = "TOBIA",
  A = "tmp_slope",
  B = "incl_rst",
  C = "imm_rst",
  D = "tmp_aspect_tot", overwrite=True)  

  grass.mapcalc("${out} = if(${A} > 0.7071,1,null())",
  out = "tmp_cataclinal",
  A = "cosmap", overwrite=True)  
  
  grass.mapcalc("${out} = if(${A} <- 0.7071,2,null())",
  out = "tmp_anaclinal",
  A = "cosmap", overwrite=True)  

  grass.mapcalc("${out} = if(${A} < 0.7071 && ${A} > -0.7071,3,null())",
  out = "tmp_othoclinal",
  A = "cosmap", overwrite=True)  

  grass.mapcalc("${out} = if(${A},if(${B} < 0.99,if(${C} < ${D},4,null()),null()),null())",
  out = "tmp_cataclinal_overdip",
  A = "tmp_cataclinal",
  B = "TOBIA",
  C = "incl_rst",
  D = "tmp_slope", overwrite=True)  

  grass.mapcalc("${out} = if(${A},if(${B} < 0.99,if(${C}>${D},5,null()),null()),null())",
  out = "tmp_cataclinal_underdip",
  A = "tmp_cataclinal",
  B = "TOBIA",
  C = "incl_rst",
  D = "tmp_slope", overwrite=True) 

  grass.mapcalc("${out} = if(${A},if(${B} > 0.99,6,null()))",
  out = "tmp_cataclinal_dip",
  A = "tmp_cataclinal",
  B = "TOBIA", overwrite=True) 
  
  grass.run_command("r.patch", input="tmp_anaclinal,tmp_othoclinal,tmp_cataclinal_overdip,tmp_cataclinal_underdip,tmp_cataclinal_dip", output="tmp_assetto", overwrite=True)
  
  grass.run_command("r.colors", map="tmp_assetto", color="rainbow") 
  
  rast="incl_rst,"+inc_map
  grass.run_command("g.copy", rast=rast)
  rast="imm_rst,"+imm_map  
  grass.run_command("g.copy", rast=rast)
  rast="cosmap,"+cos_map
  grass.run_command("g.copy", rast=rast)
  rast="tmp_assetto,"+morphostructural_map
  grass.run_command("g.copy", rast=rast)

  grass.run_command("g.mremove", flags="f", rast="tmp_*")
  grass.run_command("g.remove", rast="MASK")


if __name__ == "__main__":
  options, flags = grass.parser()
  atexit.register(cleanup)
  sys.exit(main())	
